"""
This is the tool for handling the keywords

$Id: SmartCsvExporterTool.py 12 2010-07-12 15:05:42Z gael.le-mignot $
"""

from Globals import InitializeClass
from OFS.SimpleItem import SimpleItem
from AccessControl import ClassSecurityInfo
from OFS.PropertyManager import PropertyManager
from AccessControl import getSecurityManager, Unauthorized

import csv, time
from cStringIO import StringIO

from Products.CMFCore.utils import UniqueObject, getToolByName
import Products.SmartCsvExporter.security as permissions

ID = 'portal_smart_csv_exporter'
TITLE = 'Smart CSV Exporter'
META_TYPE = 'SmartCsvExporterTool'


class SmartCsvExporterTool(UniqueObject, SimpleItem, PropertyManager):
    """
    This is the tool for handling the keywords
    """
    id = ID
    title = TITLE
    meta_type = META_TYPE

    ## Define properties
    _properties = ({'id': 'csv_options', 'type': 'lines', 'mode': 'rw', "label":"CSV writer additionnal options (option:[type:]value) type may be int, float or csv (for csv module constants)"},)

    csv_options = ["quoting:csv:QUOTE_ALL",]

    def _get_csv_options(self):
        """ get csv options as a dict from csv_options property """
        result = dict()
        for o in self.getProperty('csv_options'):
            optInfo = o.split(':')
            if len(optInfo) > 2:
                optType = optInfo.pop(1)
                if optType == 'int':
                    optInfo[1] = int(optInfo[1])
                if optType == 'floar':
                    optInfo[1] = float(optInfo[1])
                if optType == 'csv':
                    optInfo[1] = csv.__dict__[optInfo[1]]
            if o:
               result[optInfo[0]] = optInfo[1]
        return result


    manage_options = (PropertyManager.manage_options
                      + SimpleItem.manage_options)

    ## Security shortcuts
    security = ClassSecurityInfo()

    security.declarePrivate("export_csv")
    def export_csv(self, name, data):
        """
        Do a CSV export from a Python list
        """
        buffer = StringIO()
        options = self._get_csv_options()
           
        writer = csv.writer(buffer, **options)
        for row in data:
            writer.writerow(row)
        value = buffer.getvalue()
        value = unicode(value, "utf-8").encode("iso-8859-1", "replace")
        self.REQUEST.RESPONSE.setHeader('Content-Type', 'text/csv')
        self.REQUEST.RESPONSE.setHeader('Content-Disposition', 'attachment;filename=%s-%s.csv' % (name, time.strftime("%Y%m%d-%H%M")))
        return value

    security.declareProtected("View", "export_as_csv")
    def export_as_csv(self):
        """
        Modify keywords of object
        """


        parent = self.aq_parent

        # security check on object
        user = getSecurityManager().getUser()
        if not user.has_permission(permissions.EXPORT_CSV_PERMISSION, parent):
            raise Unauthorized

        fields = parent.getCustomViewFields()

        vocab = parent.listMetaDataFields(False)
        data = [ [ vocab.getValue(field, field) for field in fields ] ]
        
        items = parent.queryCatalog()
        for item in items:
            line = [ getattr(item, field, "") for field in fields ]
            # make entries exportable
            line = [self._processEntry(e) for e in line]
            data.append(line)

        return self.export_csv(parent.getId(), data)
        

    def _processEntry(self, entry):
        """
        some processing to clean up entries
        """
        if not entry:
            return ''
        
        # normalize to list
        result = []
        if not isinstance(entry, (list, tuple)):
            entry = [entry,]
        for e in entry:
            if e is None:
                e = ''
            elif not isinstance(e, str) and hasattr(e, 'Title'):
                e = e.Title()
            elif isinstance(e, unicode):
                e = e.encode('utf-8')
            elif not isinstance(e, str):
                e = str(e)
            result.append(e)
        return "\n".join(result)
                

InitializeClass(SmartCsvExporterTool)
